from argparse import ArgumentParser
import json
import sys

from aws_ip_ranges import output, data


IP_RANGES_URL = 'https://ip-ranges.amazonaws.com/ip-ranges.json'


def parse_arguments(argv):
    """Parse CLI arguments.
    """
    parser = ArgumentParser()

    parser.add_argument(
        '-k', '--yaml-key', default='aws::prefixes',
        help='Top-level key in generated YAML document. Default: %(default)s')
    source_group = parser.add_mutually_exclusive_group()
    source_group.add_argument(
        '-f', '--ip-ranges-file', metavar='PATH',
        help='Load data from a file.')
    source_group.add_argument(
        '-u', '--ip-ranges-url', metavar='URL',
        help='Load data from a custom URL. Default: %s' % IP_RANGES_URL)

    parser.add_argument(
        'output_file', nargs='?',
        help='When defined, YAML is written to this path. '
             'By default, it is written on stdout.')

    return parser.parse_args(argv)


def main(argv=None, command=None):
    """CLI main entry point.
    """
    arguments = parse_arguments(argv)

    if arguments.ip_ranges_file:
        str_data = data.load(arguments.ip_ranges_file)
    else:
        str_data = data.download(arguments.ip_ranges_url or IP_RANGES_URL)

    json_data = json.loads(str_data)
    prefixes = data.transform(json_data)
    yaml_str = output.generate(
        prefixes, json_data['createDate'], arguments.yaml_key, command)

    if arguments.output_file:
        with open(arguments.output_file, 'w') as fileobj:
            fileobj.write(yaml_str)
    else:
        sys.stdout.write(yaml_str)



import codecs
from collections import defaultdict

import requests
from netaddr import IPSet


KEYS = {
    4: {'prefixes': 'prefixes', 'prefix': 'ip_prefix'},
    6: {'prefixes': 'ipv6_prefixes', 'prefix': 'ipv6_prefix'},
}


def download(url):
    """Retrieve the AWS IP ranges data from the Internet.
    """
    return requests.get(url).text


def load(filepath, encoding='UTF-8'):
    """Read the AWS IP ranges data from a local file.
    """
    with codecs.open(filepath, encoding=encoding) as fileobj:
        return fileobj.read()


def transform(data):
    """Convert raw AWS IP ranges data to internal representation.
    """
    # Build a dict using 3 levels: region, service, ip_version
    prefixes = defaultdict(
        lambda: defaultdict(lambda: {4: IPSet(), 6: IPSet()}))

    for ip_version, ip_version_keys in KEYS.items():
        for prefix_data in data[ip_version_keys['prefixes']]:
            region = str(prefix_data['region'])
            service = str(prefix_data['service'])
            prefix = prefix_data[ip_version_keys['prefix']]
            prefixes[region][service][ip_version].add(prefix)

    # Remove service-specific prefixes from the "AMAZON" service
    for region, region_data in prefixes.items():
        for service, service_data in region_data.items():
            for ip_version, service_prefixes in service_data.items():
                if service != 'AMAZON':
                    amazon_prefixes = prefixes[region]['AMAZON'][ip_version]
                    prefixes[region]['AMAZON'][ip_version] = \
                        amazon_prefixes - service_prefixes

    return prefixes



from collections import defaultdict
from os.path import basename
import sys

from netaddr import IPSet
import yaml


TEMPLATE = """\
# Generated with "{command}"
# AWS creation date: {creation_date}
"""


class Dumper(yaml.Dumper):  # pylint: disable=too-many-ancestors
    """Dump data containing a ``netaddr.IPSet``.
    """

    def __init__(self, *args, **kwargs):
        """Register the custom representers.
        """
        super(Dumper, self).__init__(*args, **kwargs)
        self.yaml_representers[IPSet] = Dumper.represent_ipset
        self.yaml_representers[defaultdict] = Dumper.represent_defaultdict

    def process_tag(self):
        """Do nothing, because we don't want tags in the serialized document.
        """

    def represent_ipset(self, ipset):
        """Represent a ``netaddr.IPSet`` as a regular list.
        """
        return self.represent_sequence(
            'ipset', [str(network) for network in sorted(ipset._cidrs.keys())])

    def represent_defaultdict(self, dict_):
        """Represent a ``defaultdict`` as a regular dict.
        """
        return self.represent_mapping('defaultdict', dict_)


def generate(prefixes, creation_date, top_level_key, command=None):
    """Generate the YAML document.
    """
    yaml_str = TEMPLATE.format(
        command=command or ' '.join([basename(sys.argv[0])] + sys.argv[1:]),
        creation_date=creation_date)
    yaml_str += yaml.dump(
        {top_level_key: prefixes},
        indent=2, width=80, default_flow_style=False, Dumper=Dumper)

    return yaml_str



from os.path import join, dirname

from setuptools import setup, find_packages


def read(filename):
    with open(join(dirname(__file__), filename)) as fileobj:
        return fileobj.read()


def get_version(package):
    return [
        line for line in read('{}/__init__.py'.format(package)).splitlines()
        if line.startswith('__version__ = ')][0].split("'")[1]


PROJECT_NAME = 'aws-ip-ranges'
VERSION = get_version('aws_ip_ranges')


setup(
    name=PROJECT_NAME,
    version=VERSION,
    description='Retrieves and processes AWS IP ranges data.',
    author='Philippe Muller',
    author_email='philippe.muller@gmail.com',
    url='https://github.com/pmuller/aws-ip-ranges',
    packages=find_packages(exclude=['tests']),
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, <4',
    install_requires=[
        'netaddr >=0.7.19',
        'PyYAML >=3.12',
        'requests >=2.18.4',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: System :: Networking',
        'Intended Audience :: System Administrators',
    ],
    entry_points="""
        [console_scripts]
        aws-ip-ranges = aws_ip_ranges.cli:main
    """,
    license='Apache License 2.0',
)
